## ---- eval=FALSE---------------------------------------------------------
## ## Extract just the R-code for the generation of the graphics used for the figures seperately.
## ## Check if you need to get the devtools-package:
## install.knitr <- ("knitr" %in% rownames(installed.packages()) == FALSE)
## 
## if (install.knitr) {
##   print('Need to install the knitr package')
##   ## You need online access.
##     install.packages('knitr')
## }
## library(knitr)
## purl('~/git/esd_Rmarkdown/CixPAG/spell-statistics.Rmd', output='~/git/esd_Rmarkdown/CixPAG/spell-statistics.R')

## ----setup, include=FALSE------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ------------------------------------------------------------------------
## Check if you need to get the devtools-package:
install.devtools <- ("devtools" %in% rownames(installed.packages()) == FALSE)

if (install.devtools) {
  print('Need to install the devtools package')
  ## You need online access.
    install.packages('devtools')
}

## Use the devtools-package for simple facilitation of installing.
library('devtools')
install_github('metno/esd')
library(esd)
library(MASS)

## ------------------------------------------------------------------------
## Check if you need to get the devtools-package:
install.latticekrig <- ("LatticeKrig" %in% rownames(installed.packages()) == FALSE)

if (install.latticekrig) {
  print('Need to install the LatticeKrig package')
  ## You need online access.
    install.packages('LatticeKrig')
}

## ----tmax, warning=FALSE,fig.height=8,fig.width=8------------------------
if (!file.exists("tmax.india.rda")) {
  ss <- select.station(param='tmax',src='GHCND',cntr='India',nmin=50)
  tmax <- station(ss)
  map(tmax,FUN='nv',new=FALSE)
  save(tmax,file='tmax.india.rda')
} else load("tmax.india.rda")

## ----tmin, warning=FALSE,fig.height=8,fig.width=8------------------------
library(esd)
if (!file.exists("tmin.india.rda")) {
  ss <- select.station(param='tmin',src='GHCND',cntr='India',nmin=50)
  tmin <- station(ss)
  map(tmin,FUN='nv',new=FALSE)
  save(tmin,file='tmin.india.rda')
} else load("tmin.india.rda")

## ----data-quality-control------------------------------------------------
diagnose(tmax)
## Remove periods with mostly missing data and stations with few valid data
Y <- subset(tmax,it=c(1970,2015))
nv <- apply(coredata(Y),2,FUN='nv')       ##nv = number of valid data points
Y <- subset(Y,is=nv > 10290)

## ---- warning=FALSE, message=FALSE, fig.width=8, fig.height=8------------
d <- dim(Y)
n.yrs <- diff(range(year(Y)))+1
f.gt.5 <- rep(NA,d[2])        ## The portion of hot days with duration longer than five days
nf.gt.5 <- rep(NA,d[2])       ## The portion of seasons with a 5-day or longer 35C heatwave 
Pr.Tmax.gt.35<- rep(NA,d[2])  ## The probability of temperatures  greater than 35C
for (i in 1:d[2]) {
  sds <- spell(subset(Y,is=i),threshold=35,upper=30)
  if (i==1) lws <- subset(sds,is=1) else
            lws <- combine.stations(lws,subset(sds,is=1))
  ## Example of the spell statistics
  if (i==4) hist(sds,new=FALSE)
  ## Heatwaves in February-March-April 
  heatwave <- subset(sds,is=1,it=month.abb[2:4])
  ## The fraction of events lasting more than 5 days
  #f.gt.5[i] <- sum(heatwave > 5)/90
  f.gt.5[i] <- sum(heatwave > 5)/sum(is.finite(heatwave))
  #f.gt.5[i] <- sum(subset(sds,is=1) > 5)/sum(is.finite(subset(sds,is=1)))
  #f.gt.5[i] <- sum(subset(sds,is=1) > 5)/count(subset(Y,is=i),threshold=35)
  cc <- coredata(heatwave); cc[cc <= 5] <- NA; cc[is.finite(cc)] <- 1
  hw <- zoo(cc[is.finite(cc)],order.by=index(heatwave)[is.finite(cc)])
  ## The portion of seasons with a 5-day or longer 35C heatwave
  nf.gt.5[i] <- length(rownames(table(year(hw))))/n.yrs
  ## Probability of a 5-day or longer 35C heatwave in February-March-April
  Pr.Tmax.gt.35[i] <- 1-pnorm(35,mean=mean(subset(Y,it=month.abb[2:4],is=i),na.rm=TRUE),
                                 sd=sd(subset(Y,it=month.abb[2:4],is=i),na.rm=TRUE))
  print(c(f.gt.5[i],nf.gt.5[i],Pr.Tmax.gt.35[i]))
}

## ------------------------------------------------------------------------
## The mean statistics for the winter-spring, February-March-April 
tmax.fma <- aggregate(subset(Y,it=month.abb[2:4]),year,FUN='mean')
lws.fma <- aggregate(subset(lws,it=month.abb[2:4]),year,FUN='mean')
rng.tmax.fma <- range(c(tmax.fma),na.rm=TRUE) ## Sanity check
rng.lws.fma <- range(c(lws.fma),na.rm=TRUE) ## Sanity check

## Test geometric distribution for 1 day - same as the frequency of hot days?
y <- 1-pgeom(0,1/apply(lws.fma,2,'mean',na.rm=TRUE))
x <- apply(coredata(subset(Y,it=month.abb[2:4])),2,function(x) sum(x > 35,na.rm=TRUE)/sum(is.finite(x)))
plot(x,y,xlim=c(0,1),ylim=c(0,1),
     xlab=expression(sum(T > 35)/n),ylab=expression(Pr(L>1)),main='Test probability for 1 hot day')
grid()

## There are some missing data which will cause some technical problems in the analysis
## but these are only a few data points, and we can interpolate their values in order to  
## get around the stumbling blocks that the missing values cause. The function 'pcafill' makes 
## use of a spatio-temporal covariance matrix for filling in the gaps. 
tmax.fma <- pcafill(tmax.fma)
lws.fma <- pcafill(lws.fma)
## Fix suspect data caused by interpolation 
coredata(tmax.fma)[coredata(tmax.fma) > max(rng.tmax.fma)] <-  max(rng.tmax.fma)
coredata(tmax.fma)[coredata(tmax.fma) < min(rng.tmax.fma)] <-  min(rng.tmax.fma)
coredata(lws.fma)[coredata(lws.fma) > max(rng.lws.fma)] <-  max(rng.lws.fma)
coredata(lws.fma)[coredata(lws.fma) < min(rng.lws.fma)] <-  min(rng.lws.fma)

## ----number-of-heat-waves, fig.width=8, fig.height=8---------------------
## Use without pcafill - the number of hot events
lws5d <- subset(lws,it=month.abb[2:4]); x <- coredata(lws5d); x[x < 5] <- NA; x -> coredata(lws5d)
nh <- aggregate(lws5d,year,FUN='count')
attr(nh,'variable') <- 'heatwave-frequency'
attr(nh,'unit') <- 'count'
map(nh,FUN='mean',new=FALSE)
mnh <- round(apply(coredata(nh),2,'mean',na.rm=TRUE),2)
mTx <- round(apply(coredata(tmax.fma),2,'mean',na.rm=TRUE),2)
write.table(cbind(loc(lws),alt(lws),mTx,mnh),sep='  & ',eol='  \\\\  \n',quote=FALSE,row.names=FALSE)

## ------------------------------------------------------------------------
mtmax <- colMeans(coredata(tmax.fma),na.rm=TRUE)
srt <- order(mtmax)

## Use GLM and assume a Poisson distribution
cal.nevents <- data.frame(y=c(coredata(nh)),x=c(coredata(tmax.fma)))
fit.nh.glm <- glm(y ~ x, data=cal.nevents,family='poisson')

plot(cal.nevents$x,cal.nevents$y,
     xlab=expression(bar(T[max])),ylab=expression(n[h5]))
#lines(cal.nevents$x,exp(predict(fit.nh)),col='red')
## OLR and not GLM:
srt <- order(cal.nevents$x)
lines(cal.nevents$x[srt],exp(predict(fit.nh.glm))[srt],col='red')
print(summary(fit.nh.glm))

## ------------------------------------------------------------------------
mtmax <- colMeans(coredata(tmax.fma),na.rm=TRUE)
srt <- order(mtmax)

## The scatter seems to work as well with an ordinary linear model (OLR)
## Use an ordinary linear model since it's simpler than the GLM making use of 
## the mean number of events rather than an integer number.
cal.nevents <- data.frame(y=c(coredata(mnh)[srt]),x=c(mtmax[srt]))
fit.nh <- lm(y ~ x, data=cal.nevents)
plot(cal.nevents$x,cal.nevents$y,
     xlab=expression(bar(T[max])),ylab=expression(bar(n[h5])))
#lines(cal.nevents$x,exp(predict(fit.nh)),col='red')
## OLR and not GLM:
lines(cal.nevents$x,predict(fit.nh),col='red')
print(summary(fit.nh))

## ------------------------------------------------------------------------
qqnorm(coredata(mnh))
qqline(coredata(mnh),col='red',lty=2)


## ------------------------------------------------------------------------
pr.heatwave <- 1 - ppois(0,lambda=coredata(mnh))
print(round(100*pr.heatwave,2))

## ----fig2, error=FALSE,message=FALSE,warning=FALSE-----------------------
## Figure 2.
print('Figure 2')
## Synchronise the two records: mean spell length and mean temperature
xy <- merge(round(zoo(lws.fma)),zoo(tmax.fma),all=FALSE)
## In this data.frame, x is the mean heatwave duration and y is the mean temperature
cal.tmax.lws <- data.frame(x=c(coredata(tmax.fma)),y=c(coredata(lws.fma)))
#cal.tmax.lws <- data.frame(y=c(coredata(xy[,1:35])),x=c(coredata(xy)[,36:70]))
ok <- is.finite(cal.tmax.lws$x) & is.finite(cal.tmax.lws$y) & (cal.tmax.lws$y >= 0)
cal.tmax.lws <- cal.tmax.lws[ok,]
cal.tmax.lws.log <- data.frame(y=log(c(coredata(xy[,1:35]))),x=log(c(coredata(xy)[,36:70])))
ok <- is.finite(cal.tmax.lws.log$x) & is.finite(cal.tmax.lws.log$y)
cal.tmax.lws.log <- cal.tmax.lws.log[ok,]
summary(lm(y ~ x, data=cal.tmax.lws.log))
fit <- glm(y ~ x, data=cal.tmax.lws,family='poisson')
#dev.new()
par(mar=c(5.1,5.1,4.1,2.1))
plot(cal.tmax.lws$x,cal.tmax.lws$y,xlab=expression(bar(T[max])),ylab=expression(bar(L[H])))
pre <- data.frame(x=seq(min(tmax,na.rm=TRUE),max(tmax,na.rm=TRUE),by=0.1))
lines(pre$x,exp(predict(fit,newdata=pre)),col=rgb(1,0,0,0.4),lwd=3)
grid()
dev.copy2pdf(file='fig2.pdf')

## ----fig2z, error=FALSE,message=FALSE,warning=FALSE----------------------
## Figure 2.
print('Figure 2z')
## Synchronise the two records: mean spell length and mean temperature
cal.tmax.lws <- data.frame(x=c(coredata(tmax.fma)),y=1/c(coredata(lws.fma)))

fit <- glm(y ~ x, data=cal.tmax.lws,family=negative.binomial(1))
print(summary(fit))
#dev.new()
par(mar=c(5.1,5.1,4.1,2.1))
plot(cal.tmax.lws$x,1/cal.tmax.lws$y,xlab=expression(bar(T[max])),ylab=expression(bar(L[H])))
pre <- data.frame(x=seq(min(tmax,na.rm=TRUE),max(tmax,na.rm=TRUE),by=0.1))
lines(pre$x,1/exp(predict(fit,newdata=pre)),col=rgb(1,0,0,0.4),lwd=3)
grid()
dev.copy2pdf(file='fig2z.pdf')

## ----fig2x, error=FALSE,message=FALSE,warning=FALSE----------------------
## Figure 2.
print('Figure 2x')
## Synchronise the two records: mean spell length and mean temperature
xy <- merge(round(zoo(lws.fma)),zoo(tmax.fma),all=FALSE)
## In this data.frame, x is the mean heatwave duration and y is the mean temperature
cal.tmax.lws1 <- data.frame(x=apply(coredata(tmax.fma),2,'mean',na.rm=TRUE),
                           y=apply(coredata(lws.fma),2,'mean',na.rm=TRUE))
#cal.tmax.lws <- data.frame(y=c(coredata(xy[,1:35])),x=c(coredata(xy)[,36:70]))
ok <- is.finite(cal.tmax.lws1$x) & is.finite(cal.tmax.lws1$y) & (cal.tmax.lws1$y >= 0)
cal.tmax.lws1 <- cal.tmax.lws1[ok,]
fit1 <- lm(y ~ x, data=cal.tmax.lws1)
print(summary(fit1))
#dev.new()
par(mar=c(5.1,5.1,4.1,2.1))
plot(cal.tmax.lws1$x,cal.tmax.lws1$y,xlab=expression(bar(T[max])),ylab=expression(bar(L[H])),
     main='Average duration for location v.s. average temperature')
pre1 <- data.frame(x=seq(min(tmax,na.rm=TRUE),max(tmax,na.rm=TRUE),by=0.1))
lines(pre$x,exp(predict(fit1,newdata=pre1)),col=rgb(1,0,0,0.4),lwd=3)
grid()
dev.copy2pdf(file='fig2x.pdf')

## ---- pca-lws, fig.width=8, fig.height=8---------------------------------
##PCA for the mean length of warm spells:
pca.lws <- PCA(lws.fma,n=5)
plot(pca.lws,new=FALSE)

## ---- check-lws, fig.width=8, fig.height=8-------------------------------
print(summary(coredata(lws)))
print(summary(coredata(lws.fma)))
map(lws.fma,FUN='max',new=FALSE)
map(lws.fma,FUN='min',new=FALSE)

## ---- check-lws-ts, fig.width=8, fig.height=8----------------------------
plot(lws.fma,errorbar=FALSE,new=FALSE,map.show=FALSE)

## ---- pca-tmax, fig.width=8, fig.height=8--------------------------------
## PCA for the mean daily maximum temperature:
pca.tmax <- PCA(tmax.fma,n=5)
plot(pca.tmax,new=FALSE)

## ----RCP45---------------------------------------------------------------
print('Downscaling')
if (!file.exists("dse.tmax.india.rcp45.rda")) {
  T2m <- retrieve("~/data/air.mon.mean.nc",lon=c(65,95),lat=c(5,27))
  T2m <- aggregate(subset(T2m,it=month.abb[2:4]),year)
  dse.tmax.india.rcp45 <- DSensemble.pca(pca.tmax,predictor=T2m,biascorrect = TRUE,ip=1:4,it=month.abb[2:4])
  save(dse.tmax.india.rcp45,file="dse.tmax.india.rcp45.rda")
} else load("dse.tmax.india.rcp45.rda")

## ----RCP85---------------------------------------------------------------
## ----dsensemblehigh------------------------------------------------------
if (!file.exists("dse.tmax.india.rcp85.rda")) {
  T2m <- retrieve("~/data/air.mon.mean.nc",lon=c(65,95),lat=c(5,27))
  T2m <- aggregate(subset(T2m,it=month.abb[2:4]),year)
  dse.tmax.india.rcp85 <- DSensemble.pca(pca.tmax,predictor=T2m,rcp='rcp85',biascorrect = TRUE,ip=1:4,it=month.abb[2:4])
  save(dse.tmax.india.rcp85,file="dse.tmax.india.rcp85.rda")
} else load("dse.tmax.india.rcp85.rda")

## ----RCP26---------------------------------------------------------------
## ----dsensemblelow-------------------------------------------------------
if (!file.exists("dse.tmax.india.rcp26.rda")) {
  T2m <- retrieve("~/data/air.mon.mean.nc",lon=c(65,95),lat=c(5,27))
  T2m <- aggregate(subset(T2m,it=month.abb[2:4]),year)
  dse.tmax.india.rcp26 <- DSensemble.pca(pca.tmax,predictor=T2m,rcp='rcp26',biascorrect = TRUE,ip=1:4,it=month.abb[2:4])
  save(dse.tmax.india.rcp26,file="dse.tmax.india.rcp26.rda")
} else load("dse.tmax.india.rcp26.rda")

## ------------------------------------------------------------------------
## Plot the ensemble mean for all stations
plot(as.station(as.station(dse.tmax.india.rcp45)),new=FALSE,map.show=FALSE)

## ---- DS-trends, fig.width=8, fig.height=8-------------------------------
map(as.station(as.station(dse.tmax.india.rcp45)),FUN='trend',new=FALSE)
map(as.station(as.station(dse.tmax.india.rcp45)),FUN='sd',new=FALSE)
trends <- apply(as.station(as.station(dse.tmax.india.rcp45)),2,FUN='trend.coef')
sds <- apply(as.station(as.station(dse.tmax.india.rcp45)),2,FUN='sd')
names(trends) <- loc(dse.tmax.india.rcp45$pca)
names(sds) <- loc(dse.tmax.india.rcp45$pca)
print('Ensemble mean trends')
print(trends)
print('Ensemble mean sd')
print(sds)

## ---- DSE-diagnose, fig.width=8, fig.height=8----------------------------
diagnose(dse.tmax.india.rcp45,new=FALSE)

## ------------------------------------------------------------------------
crossval(dse.tmax.india.rcp45,plot=TRUE)
par(mfcol=c(1,1))

## ------------------------------------------------------------------------
plot(zoo(subset(as.station(dse.tmax.india.rcp45),is=6)),plot.type='single',
     col=rainbow(108,alpha=0.3))
grid()

## ------------------------------------------------------------------------
plot(subset(as.station(dse.tmax.india.rcp45),is=6),new=FALSE,map.show=FALSE)

## ----Mean_Feb-Apr_temperature_compared_to_hot_spell_duration-------------
## Re-calibrate the model for Indian data
print('Re-calibrate')
i1 <- is.element(loc(tmax.fma),loc(lws.fma))
i2 <- is.element(loc(lws.fma),loc(tmax.fma))
calfit <- data.frame(x=colMeans(tmax.fma,na.rm=TRUE)[i1],y=round(colMeans(lws.fma,na.rm=TRUE)[i2]))
checkfit <- data.frame(x=colMeans(tmax.fma,na.rm=TRUE)[i1],y=apply(tmax.fma,2,FUN='sd',na.rm=TRUE)[i2])
ok <- is.finite(calfit$x) & is.finite(calfit$y) & (calfit$y >= 0)
calfit <- calfit[ok,]
attr(calfit,'max(x)') <- colMeans(tmax.fma,na.rm=TRUE)[i1]
#fit <- lm(y ~ I(x) + I(x^2),data=calfit)
fit <- glm(y ~ x,data=calfit,family='poisson')
#fit <- glm(y ~ x,data=calfit,family='binomial')
#dev.new()
plot(calfit,xlab=expression(bar(T[max])),ylab=expression(bar(L[H])))
srt <- order(calfit$x)
lines(calfit$x[srt],exp(predict(fit))[srt],col='red')

## ----test-assumption-----------------------------------------------------
## Test assumption: the variance is not systematically affeced bythe mean:
plot(checkfit$x,checkfit$y,xlab=expression(mu),ylab=expression(sigma),
     main='Check for dependency between mean and variance')
abline(lm(y ~x,data=checkfit),col='red',lty=2)
grid()
## Summary of regression analysis between the seasonal mean temperature and seasonal standard deviaiton 
print(summary(lm(y ~x, data=checkfit)))

## ------------------------------------------------------------------------
## Use the ensemble mean for each station (hence repeat 'as.station' twice) for best estimate of the future temperature. This code picks a random station - the sixth site in the list - is=6
dse.tmax.is6.45 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp45)),is=6) ))) )
mnh.is6.45 <- predict(fit.nh,newdata=dse.tmax.is6.45)
pr.heatwave.is6.45 <- 1 - ppois(0,lambda=coredata(mnh.is6.45))
print(pr.heatwave.is6.45)
dse.tmax.is6.26 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp26)),is=6) ))) )
mnh.is6.26 <- predict(fit.nh,newdata=dse.tmax.is6.26)
pr.heatwave.is6.26 <- 1 - ppois(0,lambda=coredata(mnh.is6.26))
dse.tmax.is6.85 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp85)),is=6) ))) )
mnh.is6.85 <- predict(fit.nh,newdata=dse.tmax.is6.85)
pr.heatwave.is6.85 <- 1 - ppois(0,lambda=coredata(mnh.is6.85))

## ----example_downscaling_for_one_station_RCP4.5--------------------------
print('example station')
dse.is6 <- subset(as.station(dse.tmax.india.rcp45),is=6)
mwl.is6 <- dse.is6; pdf.is6 <- dse.is6;
for (i in 1:dim(pdf.is6)[2]) {
  mwl.is6[,i] <- exp(predict(fit,newdata=data.frame(x=dse.is6[,i])))
  pdf.is6[,i] <- pgeom(q=5,prob=1/(mwl.is6[,i]),lower.tail=FALSE)
}
attr(mwl.is6,'variable') <- 'heatwave-duration'
attr(mwl.is6,'unit') <- 'days'

## The probability of a 5-day long hot episode given the mean temperature (divide by 2 since q50 has been used)
pbad <- zoo(100*apply(coredata(pdf.is6),1,FUN=quantile,probs=0.5),order.by=index(dse.is6)) * 0.5
class(pbad) <- class(Y)
pbad <- attrcp(subset(Y,is=6),pbad)
attr(pbad,'variable') <- 'Pr(L > 5 days)'
attr(pbad,'unit') <- '%'

## ----example_downscaling_for_one_station_RCP8.5--------------------------
## ------------------------------------------------------------------------
## High scenarion RCP8.5:
dse.is6h <- subset(as.station(dse.tmax.india.rcp85),is=6)
mwl.is6h <- dse.is6h; pdf.is6h <- dse.is6h
for (i in 1:dim(pdf.is6h)[2]) {
  mwl.is6h[,i] <- exp(predict(fit,newdata=data.frame(x=dse.is6h[,i])))
  pdf.is6h[,i] <- pgeom(q=5,prob=1/(mwl.is6h[,i]),lower.tail=FALSE)
}

## Plot the probability of a 5-day long hot episode (divide by 2 since q50 has been used)
pbadh <- zoo(100*apply(coredata(pdf.is6h),1,FUN=quantile,probs=0.5),order.by=index(dse.is6h)) * 0.5
class(pbadh) <- class(Y)
pbadh <- attrcp(subset(Y,is=6),pbadh)
attr(pbadh,'variable') <- 'Pr(L > 5 days)'
attr(pbadh,'unit') <- '%'

## ----example_downscaling_for_one_station_RCP2.6--------------------------
## ------------------------------------------------------------------------
## Low scenario RCPO 2.6:
dse.is6l <- subset(as.station(dse.tmax.india.rcp26),is=6)
mwl.is6l <- dse.is6l; pdf.is6l <- dse.is6l
for (i in 1:dim(pdf.is6l)[2]) {
  mwl.is6l[,i] <- exp(predict(fit,newdata=data.frame(x=dse.is6l[,i])))
  pdf.is6l[,i] <- pgeom(q=5,prob=1/(mwl.is6l[,i]),lower.tail=FALSE)
}

## Plot the probability of a 5-day long hot episode (divide by 2 since q50 has been used)
pbadl <- zoo(100*apply(coredata(pdf.is6l),1,FUN=quantile,probs=0.5),order.by=index(dse.is6l)) * 0.5
class(pbadl) <- class(Y)
pbadl <- attrcp(subset(Y,is=6),pbadl)
attr(pbadl,'variable') <- 'Pr(L > 5 days)'
attr(pbadl,'unit') <- '%'

## ----Figure4a------------------------------------------------------------
## Figure 4a
## Plot the probability of one or more events with daily maximum temperature 
## above 35 degrees C lasting more than five days in the February-April season
pr.heatwave.is6.45 <- zoo(pr.heatwave.is6.45,order.by=index(pbadh))
pr.heatwave.is6.26 <- zoo(pr.heatwave.is6.26,order.by=index(pbadh))
pr.heatwave.is6.85 <- zoo(pr.heatwave.is6.85,order.by=index(pbadh))

plot(100*pr.heatwave.is6.45,main='Pr(x>0 | L > 5 days,T > 35C)',
     col=rgb(0.5,0,0,0.2),ylim=c(50,100),ylab='%',lwd=2,new=FALSE)
text(index(pbadh)[10],23.5,loc(pbadh))
lines(100*pr.heatwave.is6.26,lwd=2,col=rgb(0,0.5,0,0.2))
lines(100*pr.heatwave.is6.85,lwd=2,col=rgb(0,0,0.5,0.2))
grid()
## Add trend models
lines(trend(100*pr.heatwave.is6.85,model="y ~ I(t) + I(t^2) + I(t^3) + I(t^4)"),col=rgb(0.5,0,0),lwd=3)
lines(trend(100*pr.heatwave.is6.26,model="y ~ I(t) + I(t^2) + I(t^3) + I(t^4)"),col=rgb(0.5,0.5,0),lwd=3)
lines(trend(100*pr.heatwave.is6.45,model="y ~ I(t) + I(t^2) + I(t^3) + I(t^4)"),col=rgb(0,0,0.05),lwd=3)
dev.copy2pdf(file='fig4a.pdf')

## ----Figure4b------------------------------------------------------------
## Figure 4b
## Plot the probability that a hot day (>35 degrees C) turns into a heatwave lasting more 
## more than five days in the February-April season.
plot(pbadh,main='Pr(L > 5 days | T > 35C)',map.show=FALSE,
     col=rgb(0.5,0,0,0.2),lwd=2,new=FALSE)
text(index(pbadh)[10],23.5,loc(pbadh))
lines(pbadl,lwd=2,col=rgb(0,0.5,0,0.2))
lines(pbad,lwd=2,col=rgb(0,0,0.5,0.2))
grid()
## Add trend models
lines(trend(pbadh,model="y ~ I(t) + I(t^2) + I(t^3) + I(t^4)"),col=rgb(0.5,0,0),lwd=3)
lines(trend(pbadl,model="y ~ I(t) + I(t^2) + I(t^3) + I(t^4)"),col=rgb(0.5,0.5,0),lwd=3)
lines(trend(pbad,model="y ~ I(t) + I(t^2) + I(t^3) + I(t^4)"),col=rgb(0,0,0.05),lwd=3)
dev.copy2pdf(file='fig4b.pdf')

## ----now_year2010--------------------------------------------------------
# Calculations for Table 1 - probability of at least one heatwave in a season
## Use the ensemble mean for each station (hence repeat 'as.station' twice) for best estimate of the future temperature
dse.tmax.2010.45 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp45)),it=2010) ))) )
mnh.2010.45 <- predict(fit.nh,newdata=dse.tmax.2010.45)
pr.heatwave.2010.45 <- 1 - ppois(0,lambda=coredata(mnh.2010.45))
dse.tmax.2010.26 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp26)),it=2010) ))) )
mnh.2010.26 <- predict(fit.nh,newdata=dse.tmax.2010.26)
pr.heatwave.2010.26 <- 1 - ppois(0,lambda=coredata(mnh.2010.26))
dse.tmax.2010.85 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp85)),it=2010) ))) )
mnh.2010.85 <- predict(fit.nh,newdata=dse.tmax.2010.85)
pr.heatwave.2010.85 <- 1 - ppois(0,lambda=coredata(mnh.2010.85))

# Calculations for Table 2 - probability of a hot day turning into a heatwave
dse.2010.rcp45 <- 
  subset(as.station(as.station(dse.tmax.india.rcp45,FUN='quantile',probs=0.5)),it=c(2008,2012))
newdata <- data.frame(x=c(coredata(dse.2010.rcp45)))
mlength <- exp(predict(fit,newdata=newdata))
zzz <- 100*pgeom(q=5,prob=1/(mlength),lower.tail=FALSE) * 0.5
coredata(dse.2010.rcp45) <- zzz

dse.2010.rcp26 <- 
  subset(as.station(as.station(dse.tmax.india.rcp26,FUN='quantile',probs=0.5)),it=c(2008,2012))
newdata <- data.frame(x=c(coredata(dse.2010.rcp26)))
mlength <- exp(predict(fit,newdata=newdata))
zzz <- 100*pgeom(q=5,prob=1/(mlength),lower.tail=FALSE) * 0.5
coredata(dse.2010.rcp26) <- zzz

dse.2010.rcp85 <- 
  subset(as.station(as.station(dse.tmax.india.rcp85,FUN='quantile',probs=0.5)),it=c(2008,2012))
newdata <- data.frame(x=c(coredata(dse.2010.rcp85)))
mlength <- exp(predict(fit,newdata=newdata))
zzz <- 100*pgeom(q=5,prob=1/(mlength),lower.tail=FALSE) *  0.5
coredata(dse.2010.rcp85) <- zzz

## ----soon_year2050-------------------------------------------------------
# Table 1 
dse.tmax.2050.45 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp45)),it=2050) ))) )
mnh.2050.45 <- predict(fit.nh,newdata=dse.tmax.2050.45)
pr.heatwave.2050.45 <- 1 - ppois(0,lambda=coredata(mnh.2050.45))
dse.tmax.2050.26 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp26)),it=2050) ))) )
mnh.2050.26 <- predict(fit.nh,newdata=dse.tmax.2050.26)
pr.heatwave.2050.26 <- 1 - ppois(0,lambda=coredata(mnh.2050.26))
dse.tmax.2050.85 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp85)),it=2050) ))) )
mnh.2050.85 <- predict(fit.nh,newdata=dse.tmax.2050.85)
pr.heatwave.2050.85 <- 1 - ppois(0,lambda=coredata(mnh.2050.85))

# Table 2
dse.2050.rcp45 <- 
  subset(as.station(as.station(dse.tmax.india.rcp45,FUN='quantile',probs=0.5)),it=c(2058,2052))
newdata <- data.frame(x=c(coredata(dse.2050.rcp45)))
mlength <- exp(predict(fit,newdata=newdata))
zzz <- 100*pgeom(q=5,prob=1/(mlength),lower.tail=FALSE) *  0.5
coredata(dse.2050.rcp45) <- zzz

dse.2050.rcp26 <- 
  subset(as.station(as.station(dse.tmax.india.rcp26,FUN='quantile',probs=0.5)),it=c(2058,2052))
newdata <- data.frame(x=c(coredata(dse.2050.rcp26)))
mlength <- exp(predict(fit,newdata=newdata))
zzz <- 100*pgeom(q=5,prob=1/(mlength),lower.tail=FALSE) *  0.5
coredata(dse.2050.rcp26) <- zzz

dse.2050.rcp85 <- 
  subset(as.station(as.station(dse.tmax.india.rcp85,FUN='quantile',probs=0.5)),it=c(2058,2052))
newdata <- data.frame(x=c(coredata(dse.2050.rcp85)))
mlength <- exp(predict(fit,newdata=newdata))
zzz <- 100*pgeom(q=5,prob=1/(mlength),lower.tail=FALSE) * 0.5
coredata(dse.2050.rcp85) <- zzz

## ----future_year2100-----------------------------------------------------
# Table 1
dse.tmax.2100.45 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp45)),it=2099) ))) )
mnh.2100.45 <- predict(fit.nh,newdata=dse.tmax.2100.45)
pr.heatwave.2100.45 <- 1 - ppois(0,lambda=coredata(mnh.2100.45))
dse.tmax.2100.26 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp26)),it=2099) ))) )
mnh.2100.26 <- predict(fit.nh,newdata=dse.tmax.2100.26)
pr.heatwave.2100.26 <- 1 - ppois(0,lambda=coredata(mnh.2100.26))
dse.tmax.2100.85 <- data.frame( x = c(t(coredata( subset(as.station(as.station(dse.tmax.india.rcp85)),it=2099) ))) )
mnh.2100.85 <- predict(fit.nh,newdata=dse.tmax.2100.85)
pr.heatwave.2100.85 <- 1 - ppois(0,lambda=coredata(mnh.2100.85))

# Table 2
dse.2100.rcp45 <- 
  subset(as.station(as.station(dse.tmax.india.rcp45,FUN='quantile',probs=0.5)),it=c(2095,2100))
newdata <- data.frame(x=c(coredata(dse.2100.rcp45)))
mlength <- exp(predict(fit,newdata=newdata))
zzz <- 100*pgeom(q=5,prob=1/(mlength),lower.tail=FALSE) * 0.5
coredata(dse.2100.rcp45) <- zzz

dse.2100.rcp26 <- 
  subset(as.station(as.station(dse.tmax.india.rcp26,FUN='quantile',probs=0.5)),it=c(2095,2100))
newdata <- data.frame(x=c(coredata(dse.2100.rcp26)))
mlength <- exp(predict(fit,newdata=newdata))
zzz <- 100*pgeom(q=5,prob=1/(mlength),lower.tail=FALSE) * 0.5
coredata(dse.2100.rcp26) <- zzz

dse.2100.rcp85 <- 
  subset(as.station(as.station(dse.tmax.india.rcp85,FUN='quantile',probs=0.5)),it=c(2095,2100))
newdata <- data.frame(x=c(coredata(dse.2100.rcp85)))
mlength <- exp(predict(fit,newdata=newdata))
zzz <- 100*pgeom(q=5,prob=1/(mlength),lower.tail=FALSE) * 0.5
coredata(dse.2100.rcp85) <- zzz

## ----table1--------------------------------------------------------------
## Table 1
r1 <- round(cbind(100*pr.heatwave.2010.45,100*pr.heatwave.2050.45,100*pr.heatwave.2100.45))
colnames(r1) <- paste(rep('RCP 4.5',3),c('2010','2050','2100'))

r2 <- round(cbind(100*pr.heatwave.2050.26,100*pr.heatwave.2100.26))
colnames(r2) <- paste(rep('RCP 2.6',2),c('2050','2100'))

r3 <- round(cbind(100*pr.heatwave.2050.85,100*pr.heatwave.2100.85))
colnames(r3) <- paste(rep('RCP 8.5',2),c('2050','2100'))

tab1 <- cbind(round(100*(nf.gt.5)),r1,r2,r3)
colnames(tab1)[1] <- 'obs.freq'
rownames(tab1) <- substr(loc(dse.2010.rcp45),1,9)
write.table(tab1,sep='  & ',eol='  \\\\  \n',quote=FALSE)

## ----table2--------------------------------------------------------------
## Table 2
colnames(dse.2010.rcp45) <- loc(dse.2010.rcp45)
pr.45.2010 <- apply(dse.2010.rcp45,2,'mean')
colnames(dse.2050.rcp45) <- loc(dse.2050.rcp45)
pr.45.2050 <- apply(dse.2050.rcp45,2,'mean')
colnames(dse.2100.rcp45) <- loc(dse.2100.rcp45)
pr.45.2100 <- apply(dse.2100.rcp45,2,'mean')
r1 <- round(cbind(pr.45.2010,pr.45.2050,pr.45.2100))
colnames(r1) <- paste(rep('RCP 4.5',3),c('2010','2050','2100'))

colnames(dse.2010.rcp26) <- loc(dse.2010.rcp26)
pr.26.2010 <- apply(dse.2010.rcp26,2,'mean')
colnames(dse.2050.rcp26) <- loc(dse.2050.rcp26)
pr.26.2050 <- apply(dse.2050.rcp26,2,'mean')
colnames(dse.2100.rcp26) <- loc(dse.2100.rcp26)
pr.26.2100 <- apply(dse.2100.rcp26,2,'mean')
r2 <- round(cbind(pr.26.2050,pr.26.2100))
colnames(r2) <- paste(rep('RCP 2.6',2),c('2050','2100'))

colnames(dse.2010.rcp85) <- loc(dse.2010.rcp85)
pr.85.2010 <- apply(dse.2010.rcp85,2,'mean')
colnames(dse.2050.rcp85) <- loc(dse.2050.rcp85)
pr.85.2050 <- apply(dse.2050.rcp85,2,'mean')
colnames(dse.2100.rcp85) <- loc(dse.2100.rcp85)
pr.85.2100 <- apply(dse.2100.rcp85,2,'mean')
r3 <- round(cbind(pr.85.2050,pr.85.2100))
colnames(r3) <- paste(rep('RCP 8.5',2),c('2050','2100'))

tab2 <- cbind(round(100*(f.gt.5)),r1,r2,r3)
colnames(tab2)[1] <- 'obs.freq'
rownames(tab2) <- substr(rownames(tab1),1,9)
write.table(tab2,sep='  & ',eol='  \\\\  \n',quote=FALSE)

## ------------------------------------------------------------------------
west_lat <- c(28.50, 31.82, 33.19, 34.05, 29.80, 25.81, 28.50)
west_lon <- c(71.86, 74.08, 72.68, 73.72, 78.45, 74.82, 71.86)
cent_lat <- c(25.81, 29.80, 27.56, 22.05, 25.81)
cent_lon <- c(74.82, 78.45, 83.24, 80.89, 74.82)
east_lat <- c(22.05, 27.56, 26.95, 24.10, 22.13, 22.05)
east_lon <- c(80.89, 83.24, 88.00, 87.94, 84.83, 80.89)

## ------------------------------------------------------------------------
err <- abs(100*(nf.gt.5) - 100*(pr.heatwave))
print(err)
ile <- err/(100*(nf.gt.5)) > 0.5  ## identify the locations with large error
pch <- rep(19,length(err)); col <- rep('darkgreen',length(err))
pch[ile] <- 17; col[ile] <- 'orange'
plot(lon(lws),lat(lws),pch=pch,col=col,xlab='',ylab='')
text(lon(lws),lat(lws),substr(loc(lws),1,7),cex=0.7,col='grey')
data(geoborders)
lines(geoborders)

lines(west_lon,west_lat,lwd=3,col=rgb(0,0,0,0.05))
lines(cent_lon,cent_lat,lwd=3,col=rgb(0,0,0,0.05))
lines(east_lon,east_lat,lwd=3,col=rgb(0,0,0,0.05))

## ------------------------------------------------------------------------
err <- abs(100*(f.gt.5) - pr.45.2010)
print(err)
ile <- err/(100*(f.gt.5)) > 0.5  ## identify the locations with large error
pch <- rep(19,length(err)); col <- rep('darkgreen',length(err))
pch[ile] <- 17; col[ile] <- 'orange'
plot(lon(lws),lat(lws),pch=pch,col=col,xlab='',ylab='')
text(lon(lws),lat(lws),substr(loc(lws),1,7),cex=0.7,col='grey')
data(geoborders)
lines(geoborders)

lines(west_lon,west_lat,lwd=3,col=rgb(0,0,0,0.05))
lines(cent_lon,cent_lat,lwd=3,col=rgb(0,0,0,0.05))
lines(east_lon,east_lat,lwd=3,col=rgb(0,0,0,0.05))

## ----Figure5a, warning=FALSE, message=FALSE------------------------------
## Figure 5a Map of the probability of at least one event per season at 2100 RCP4.5
demo(gridmap,ask=FALSE)
attr(pr.heatwave.2100.45,'longitude') <- lon(dse.2100.rcp45)
attr(pr.heatwave.2100.45,'latitude') <- lat(dse.2100.rcp45)
attr(pr.heatwave.2100.45,'altitude') <- alt(dse.2100.rcp45)
attr(pr.heatwave.2100.45,'unit') <- esd::unit(dse.2100.rcp45)
attr(pr.heatwave.2100.45,'variable') <- varid(dse.2100.rcp45)
print(summary(pr.heatwave.2100.45))
prng0 <- gridmap(Y=100*pr.heatwave.2100.45,zlim=c(0,max(alt(tmax))+100),verbose=TRUE)
attr(prng0,'variable') <- 'Pr(x.gt.0|L.gt.5.days,T.gt.35C)'
attr(prng0,'unit') <- '"%"'
map(prng0,new=FALSE)
par(fig=c(0,1,0.1,1))
points(lon(dse.2100.rcp45),lat(dse.2100.rcp45),pch=19,col='white',cex=0.5)
points(lon(dse.2100.rcp45),lat(dse.2100.rcp45),cex=0.5)
text(lon(dse.2100.rcp45),lat(dse.2100.rcp45),substr(loc(dse.2100.rcp45),1,5),cex=0.7,pos=1)
lines(west_lon,west_lat,lwd=3,col=rgb(0,0,0,0.05))
lines(cent_lon,cent_lat,lwd=3,col=rgb(0,0,0,0.05))
lines(east_lon,east_lat,lwd=3,col=rgb(0,0,0,0.05))
dev.copy2pdf(file='fig5a.pdf')

## ----Figure5b, warning=FALSE, message=FALSE------------------------------
## Figure 5b Map of the probability of a hot day (Tmax>35C) turning into a heat wave lasting more than 5 days at 2100 RCP4.5

prgt5d <- gridmap(dse.2100.rcp45,zlim=c(0,max(alt(tmax))+100))
attr(prgt5d,'variable') <- 'Pr(L.gt.5.days|T.gt.35C)'
attr(prgt5d,'unit') <- '"%"'
map(prgt5d,new=FALSE)
par(fig=c(0,1,0.1,1))
points(lon(dse.2100.rcp45),lat(dse.2100.rcp45),pch=19,col='white',cex=0.5)
points(lon(dse.2100.rcp45),lat(dse.2100.rcp45),cex=0.5)
text(lon(dse.2100.rcp45),lat(dse.2100.rcp45),substr(loc(dse.2100.rcp45),1,5),cex=0.7,pos=1)
lines(west_lon,west_lat,lwd=3,col=rgb(0,0,0,0.05))
lines(cent_lon,cent_lat,lwd=3,col=rgb(0,0,0,0.05))
lines(east_lon,east_lat,lwd=3,col=rgb(0,0,0,0.05))
dev.copy2pdf(file='fig5b.pdf')

## ----Tmax-map------------------------------------------------------------
## 
ztmax <- gridmap(tmax)
map(ztmax,colbar=list(breaks=seq(5,45,by=1)),new=FALSE)
par(fig=c(0,1,0.1,1))
points(lon(tmax),lat(tmax),pch=19,col='white',cex=0.5)
points(lon(tmax),lat(tmax),cex=0.5)
text(lon(tmax),lat(tmax),substr(loc(tmax),1,5),cex=0.7,pos=1)

## ----Tmin-map------------------------------------------------------------
## 
ztmin <- gridmap(tmin)
map(ztmin,colbar=list(breaks=seq(5,45,by=1)),new=FALSE)
par(fig=c(0,1,0.1,1))
points(lon(tmin),lat(tmin),pch=19,col='white',cex=0.5)
points(lon(tmin),lat(tmin),cex=0.5)
text(lon(tmin),lat(tmin),substr(loc(tmin),1,5),cex=0.7,pos=1)

## ----downscaled-Tmax-map-------------------------------------------------
dsetmax.2099 <- gridmap(subset(as.station(as.station(dse.tmax.india.rcp85)),it=2099))
map(dsetmax.2099,colbar=list(breaks=seq(5,45,by=1)),new=FALSE)
par(fig=c(0,1,0.1,1))
points(lon(tmax),lat(tmax),pch=19,col='white',cex=0.5)
points(lon(tmax),lat(tmax),cex=0.5)
text(lon(tmax),lat(tmax),substr(loc(tmax),1,5),cex=0.7,pos=1)

## ----downscaled-Tmax-change-map------------------------------------------
dsetmax.2010 <- gridmap(subset(as.station(as.station(dse.tmax.india.rcp85)),it=2010))
dsetmax.2010 <- dsetmax.2099 - dsetmax.2010
map(dsetmax.2010,new=FALSE)
par(fig=c(0,1,0.1,1))
points(lon(tmax),lat(tmax),pch=19,col='white',cex=0.5)
points(lon(tmax),lat(tmax),cex=0.5)
text(lon(tmax),lat(tmax),substr(loc(tmax),1,5),cex=0.7,pos=1)

## ------------------------------------------------------------------------
## Function: autocorrelation

AR <- function(n,mean=1,sd=1,a1=0.8) {
  rn <- rnorm(n,mean=mean,sd=sd)
  for (i in 2:n) rn[i] <- (a1*rn[i-1] + (1-a1)*rn[i])
  invisible(rn)
}

## ---- warning=FALSE, message=FALSE---------------------------------------
## Script that reads European temperature data and explores the connection between 
## the seasonal mean temperature and the mean length of the warm/cold spells 

for (it in c('djf','jja')) {

  if (it =='djf') {
    cold <- TRUE
    threshold <- 0
    is <- 2
  } else {
    cold <- FALSE
    threshold <- 20
    is <- 1
  }

  ss <- select.station(src='ecad',param='tx',nmin=75)
  d <- dim(ss)
  x <- rep(NA,d[1]); y <- x; std <- y
  q.spell <- rep(NA,d[1]*10); dim(q.spell) <- c(d[1],10); q.geom <- q.spell

  if (!file.exists(paste('ecad.tg.',it,'.rda',sep=''))) {
    for (i in seq(d[1])) {
      z <- station(ss[i,])
      print(loc(z))
      ## Make sure that there are values above and below the given threshold - otherwise
      ## spell will not work.
      if ( (sum(z > threshold,na.rm=TRUE)>1000) & (sum(z < threshold,na.rm=TRUE)> 1000) ) {
        s <- spell(z,threshold=threshold)
        ## Quality check: durations longer than a season (100 days) are not credible
        sc <- coredata(s); sc[sc > 100] <- NA; sc -> coredata(s)
        y[i] <- mean(subset(subset(s,is=is),it=it),na.rm=TRUE)
    
        ## Compare the spell-distribution with a geometric distribution
        q.spell[i,] <- quantile(subset(subset(s,is=is),it=it),probs=seq(0.05,0.95,by=0.1),na.rm=TRUE)
        q.geom[i,] <- qgeom(p=seq(0.05,0.95,by=0.1),prob=1/(y[i]))
        std[i] <- sd(subset(z,it=it),na.rm=TRUE)
        x[i] <- mean(subset(z,it=it),na.rm=TRUE)
      }
    }
    save(x,y,s,std,q.spell,q.geom,file=paste('ecad.tg.',it,'.rda',sep=''))
  } else load(paste('ecad.tg.',it,'.rda',sep=''))
  ## Plot results

  x[x > 50] <- NA
  if (it=='djf') x[x>10] <- NA
  
  plot(x,y,main=paste('Mean L & mean',toupper(it),' T ',
                    c('below','above')[c(cold,!cold)],threshold,'C'),
     sub='source: ECA&D',pch=19,col=rgb(0.5,0,0,0.3),
     xlab=expression(bar(T)),ylab=expression(bar(L)))
  grid()

  ## Monte-Carlo simulations to compare spell length with 
  z <- station(ss[1,])
  mstd <- 1.5*quantile(std,0.99,na.rm=TRUE)
  if (!cold) mx <- mean(x,na.rm=TRUE) else mx <- 0
  nmc <- 300
  ymc <- rep(NA,nmc); xmc <- ymc

  for (i in 1:nmc) {
    m <- seq(mx-mstd,mx+mstd,length=nmc)[i]
    coredata(z) <- AR(length(z),mean=m,sd=mstd,a1=0.7)
    s <- spell(z,threshold=threshold)
    if (length(s) > 0) {
      ymc[i] <- mean(subset(subset(s,is=is),it=it),na.rm=TRUE)
      xmc[i] <- m
      points(xmc,ymc,pch=19,col='grey75')
    }
  }

  points(x,y,pch=19,col=rgb(0.5,0,0,0.3))

  ix <- order(x); x <- x[ix]; y <- y[ix]
  ok <- is.finite(x) & is.finite(y)
  x <- x[ok]; y <- y[ok]; std <- std[ok]
  #calfit <- data.frame(x=x,y=y)
  calfit <- data.frame(x=x[(x > -20) & (x < 35)],y=y[(x > -20) & (x < 35)]) ## Exclude the noisy tails
  calfit2 <- data.frame(x=x[(x > -20) & (x < 35)],y=1/y[(x > -20) & (x < 35)]) ## Exclude the noisy tails
  attr(calfit,'max(x)') <- max(x,na.rm=TRUE)
  #fit <- lm(y ~ I(x) + I(x^2),data=calfit)
  fit <- glm(y ~ x,data=calfit,family='poisson')
  #fit <- glm(y ~ x,data=calfit,family='binomial')
  fit2 <- glm(y ~ x, data=calfit2,family=negative.binomial(1))
  print(summary(fit))
  lines(calfit$x,exp(predict(fit)),col='red')
  lines(calfit$x,1/exp(predict(fit2)),col='red',lty=2)
  dev.copy2pdf(file=paste('fig1',c('a','b')[c(cold,!cold)],'.pdf',sep=''))
  
  attr(x,'description') <- paste(it,'mean temperature (degC)')
  if (cold) attr(y,'description') <- 'mean cold spell length (days)' else
            attr(y,'description') <- 'mean warm spell length (days)'
  attr(y,'threshold') <- threshold
  attr(x,'label') <- expression(bar(T))
  attr(x,'Monte-Carlo') <- xmc
  attr(y,'label') <- expression(bar(tau[T < T0]))
  attr(y,'Monte-Carlo)') <- ymc
  meanspell <- data.frame(meanT=x,meanL=y,std=std)
  attr(meanspell,'fit') <- fit
  attr(meanspell,'geometric.fit') <- data.frame(q.spell=q.spell,q.geom = q.geom)
  save(meanspell,file=paste('meanspell',it,c('below','above')[c(cold,!cold)],threshold,'.rda',sep=''))
  
  ## Test if the spell length statistics is close to geometric
  plot(c(q.spell),c(q.geom),main='Spell length statistics',
      xlim=range(0,90),ylim=range(0,90),pch=19,col=rgb(0,0,0,0.2),
      xlab=expression(q[p]),ylab='qgeom(p,1/mean)')
  grid()
  lines(range(q.spell,q.geom,na.rm=TRUE),range(q.spell,q.geom,na.rm=TRUE),col='red')
  dev.copy2pdf(file=paste('fig3',c('a','b')[c(cold,!cold)],'.pdf',sep=''))
}

